@@ -17,7 +17,7 @@ use Regexp::Common qw/comment delimited/;
 our $WS_RE = qr/[ \t\v\n\f]/;          # C.f. doAsmOpaque()
 our $ASM_COMMENT_RE = qr/(?:;[^\n]*|$RE{comment}{'C++'})/;
 
-our $VERSION = '0.33'; # TRIAL VERSION
+our $VERSION = '0.36'; # TRIAL VERSION
 
 
 # ----------------------------------------------------------------------------------------
@@ -32,8 +32,9 @@ sub new {
   map {$logInfo{$_}++} @{$logInfo};
 
   my $grammarName = $opts{grammarName} || 'ISO-ANSI-C-2011';
+  my $start = $opts{start} || '';
 
-  my $grammar = MarpaX::Languages::C::AST::Grammar->new($grammarName, \%logInfo);
+  my $grammar = MarpaX::Languages::C::AST::Grammar->new($grammarName, \%logInfo, $start);
   my $grammar_option = $grammar->grammar_option();
   $grammar_option->{bless_package} = 'C::AST';
   $grammar_option->{source} = \$grammar->content();
@@ -55,6 +56,18 @@ sub new {
       $lexemeCallback = shift(@lexemeCallbackArgs);
   }
 
+  my $typedef = $opts{typedef} || [];
+  if (ref($typedef) ne 'ARRAY') {
+      croak 'typedef must be a reference to ARRAY';
+  }
+
+  my $enum = $opts{enum} || [];
+  if (ref($enum) ne 'ARRAY') {
+      croak 'enum must be a reference to ARRAY';
+  }
+
+  my $lazy = $opts{lazy} || 0;
+
   my $self  = {
                _scope              => MarpaX::Languages::C::AST::Scope->new(),
                _grammar            => $grammar,
@@ -62,16 +75,39 @@ sub new {
                _sourcep            => undef,
 	       _lexemeCallback     => $lexemeCallback,
 	       _lexemeCallbackArgs => \@lexemeCallbackArgs,
-	       _logInfo            => \%logInfo
+	       _logInfo            => \%logInfo,
+	       _typedef            => $typedef,
+	       _enum               => $enum,
+	       _lazy               => $lazy,
+	       _start              => $start
               };
 
   bless($self, $class);
 
+  $self->_init();
+
   return $self;
 }
 
 # ----------------------------------------------------------------------------------------
 
+sub _init {
+    my $self = shift;
+
+    #
+    # Insert known typedef and enum at the top-level scope
+    #
+    foreach (@{$self->{_typedef}}) {
+	$self->scope->parseEnterTypedef($_, [0, length($_)]);
+    }
+    foreach (@{$self->{_enum}}) {
+	$self->scope->parseEnterEnum($_, [0, length($_)]);
+    }
+
+    return;
+}
+# ----------------------------------------------------------------------------------------
+
 sub _context {
     my $self = shift;
 
@@ -85,7 +121,7 @@ sub _context {
 
 
 sub parse {
-  my ($self, $sourcep, $optionalArrayOfValuesb) = @_;
+  my ($self, $sourcep) = @_;
 
   $self->{_sourcep} = $sourcep;
   $self->{_callbackEvents} = MarpaX::Languages::C::AST::Callback::Events->new($self);
@@ -106,11 +142,38 @@ sub parse {
       logCroak("%s", $origError);
     }
   }
+  #
+  # The following will be used by callbacks to avoid a call to lastCompleted:
+  #
+  # In Callback/Events.pm, it is clear that we need to retreive the lexeme value
+  # in only one single case: directDeclaratorIdentifier$. This can be done
+  # in a generic way using lastCompleted(), but this is cost a lot and this is not
+  # needed! In fact, if you look to the grammar you will see that IDENTIFIER is
+  # systematically paused before.It is only in _doPauseBeforeLexeme() that IDENTIFIER
+  # can setted. so if _doPauseBeforeLexeme() is setting $self->{_lastIdentifier} value
+  # everytime it is doing a lexeme_read() on IDENTIFIER, the directDeclaratorIdentifier$
+  # event will be triggered and the directDeclaratorIdentifier LHS symbol value is
+  # guaranteed to be what _doPauseBeforeLexeme() has used for its lexeme_read.
+  #
+  # This is because directDeclaratorIdentifier rule is made of only ONE rhs:
+  #
+  # directDeclaratorIdentifier ::= IDENTIFIER
+
+
+  $self->{_lastIdentifier} = undef;
   do {
     my %lexeme = ();
+    #
+    # Note 1: it is very important that neither _getLexeme() or _doScope() could
+    #         generate an event
+    #
     $self->_getLexeme(\%lexeme);
     $self->_doScope(\%lexeme);
     $self->_doEvents();
+    #
+    # Note 2: Any routine below that could generate an event must call again
+    #         _doEvents()
+    #
     $self->_doAsmOpaque(\%lexeme, $pos, $max);
     $pos += $self->_doPauseBeforeLexeme(\%lexeme);
     $self->_doLogInfo(\%lexeme);
@@ -161,13 +224,20 @@ sub value {
     logCroak('No parse tree  value.');
   }
   do {
-    $valuep = $self->{_impl}->value();
-    if (defined($valuep)) {
-	if (! $arrayOfValuesb) {
-	    logCroak('There is more than just one parse tree value.');
-	}
-      push(@rc, $valuep);
-    }
+      $valuep = $self->{_impl}->value();
+      if (defined($valuep)) {
+	  if (! $arrayOfValuesb) {
+	      if ($self->{_lazy}) {
+		  $log->infof('There is more than just one parse tree value, but lazy mode allow this.');
+		  $valuep = undef;
+	      } else {
+		  logCroak('There is more than just one parse tree value.');
+	      }
+	  }
+	  if (defined($valuep)) {
+	      push(@rc, $valuep);
+	  }
+      }
   } while (defined($valuep));
   if ($arrayOfValuesb) {
     return \@rc;
@@ -209,6 +279,8 @@ sub _getLexeme {
     ($lexemeHashp->{line}, $lexemeHashp->{column}) = $self->{_impl}->line_column($lexemeHashp->{start});
     $lexemeHashp->{value} = $self->{_impl}->literal($lexemeHashp->{start}, $lexemeHashp->{length});
   }
+
+  return;
 }
 # ----------------------------------------------------------------------------------------
 sub _doLogInfo {
@@ -219,6 +291,8 @@ sub _doLogInfo {
 	  $log->infof("[%8d:%3d] %-30s %s", $lexemeHashp->{line}, $lexemeHashp->{column}, $lexemeHashp->{name}, $lexemeHashp->{value});
       }
   }
+
+  return;
 }
 # ----------------------------------------------------------------------------------------
 sub _doLexemeCallback {
@@ -228,6 +302,8 @@ sub _doLexemeCallback {
       my $callback = $self->{_lexemeCallback};
       &$callback(@{$self->{_lexemeCallbackArgs}}, $lexemeHashp);
   }
+
+  return;
 }
 # ----------------------------------------------------------------------------------------
 sub _doPreprocessing {
@@ -300,6 +376,8 @@ sub _doPreprocessing {
 	$delta += $length;
     }
     pos(${$self->{_sourcep}}) = $previous;
+
+    return;
 }
 # ----------------------------------------------------------------------------------------
 sub _doScope {
@@ -387,6 +465,8 @@ sub _doScope {
       }
     }
   }
+
+  return;
 }
 # ----------------------------------------------------------------------------------------
 sub _doAsmOpaque {
@@ -534,6 +614,8 @@ sub _doAsmOpaque {
       pos(${$self->{_sourcep}}) = $prevpos;
     }
   }
+
+  return;
 }
 # ----------------------------------------------------------------------------------------
 sub _doPauseBeforeLexeme {
@@ -551,32 +633,66 @@ sub _doPauseBeforeLexeme {
       if ($lexemeHashp->{name} eq 'TYPEDEF_NAME' ||
           $lexemeHashp->{name} eq 'ENUMERATION_CONSTANT' ||
           $lexemeHashp->{name} eq 'IDENTIFIER') {
-	  my @terminals_expected = @{$self->{_impl}->terminals_expected()};
+	  my @alternatives = ();
 	  #
 	  # Determine the correct lexeme
 	  #
-	  my $newlexeme;
-	  if ((grep {$_ eq 'TYPEDEF_NAME'} @terminals_expected) && $self->{_scope}->parseIsTypedef($lexemeHashp->{value})) {
-	      $newlexeme = 'TYPEDEF_NAME';
-	  } elsif ((grep {$_ eq 'ENUMERATION_CONSTANT'} @terminals_expected) && $self->{_scope}->parseIsEnum($lexemeHashp->{value})) {
-	      $newlexeme = 'ENUMERATION_CONSTANT';
-	  } elsif ((grep {$_ eq 'IDENTIFIER'} @terminals_expected)) {
-	      $newlexeme = 'IDENTIFIER';
+	  if ($self->{_lazy}) {
+	      if ($self->{_scope}->parseIsTypedef($lexemeHashp->{value})) {
+		  @alternatives = qw/TYPEDEF_NAME IDENTIFIER/;
+	      } elsif ($self->{_scope}->parseIsEnum($lexemeHashp->{value})) {
+		  @alternatives = qw/ENUMERATION_CONSTANT IDENTIFIER/;
+	      } else {
+		  @alternatives = qw/TYPEDEF_NAME ENUMERATION_CONSTANT IDENTIFIER/;
+	      }
 	  } else {
+	      my %terminals_expected = map {$_ => 1} @{$self->{_impl}->terminals_expected()};
+	      if (exists($terminals_expected{TYPEDEF_NAME}) && $self->{_scope}->parseIsTypedef($lexemeHashp->{value})) {
+		  push(@alternatives, 'TYPEDEF_NAME');
+	      } elsif (exists($terminals_expected{ENUMERATION_CONSTANT}) && $self->{_scope}->parseIsEnum($lexemeHashp->{value})) {
+		  push(@alternatives, 'ENUMERATION_CONSTANT');
+	      } elsif (exists($terminals_expected{IDENTIFIER})) {
+		  push(@alternatives, 'IDENTIFIER');
+		  #
+		  # Hack for the Callback framework: store in advance the IDENTIFIER, preventing
+		  # a call to lastCompleted
+		  #
+		  $self->{_lastIdentifier} = $lexemeHashp->{value};
+	      }
+	  }
+	  if (! @alternatives) {
 	      my $line_columnp = lineAndCol($self->{_impl});
 	      logCroak("[%s] Lexeme value \"%s\" cannot be associated to TYPEDEF_NAME, ENUMERATION_CONSTANT nor IDENTIFIER at line %d, column %d.\n\nLast position:\n\n%s%s", whoami(__PACKAGE__), $lexemeHashp->{value}, $lexemeHashp->{line}, $lexemeHashp->{column}, showLineAndCol($lexemeHashp->{line}, $lexemeHashp->{column}, $self->{_sourcep}), $self->_context());
 	  }
 	  #
-	  # Push the unambiguated lexeme
+	  # Push the alternatives, more than one possible only if lazy mode is turned on
 	  #
-	  if ($log->is_debug) {
-	      $log->debugf('[%s] Pushing lexeme %s "%s"', whoami(__PACKAGE__), $newlexeme, $lexemeHashp->{value});
+	  my @alternativesOk = ();
+	  my $is_debug = $log->is_debug;
+	  foreach (@alternatives) {
+	      if (defined($self->{_impl}->lexeme_alternative($_, $lexemeHashp->{value}))) {
+		  push(@alternativesOk, $_);
+		  if ($is_debug) {
+		      $log->debugf('[%s] Pushed alternative %s "%s"', whoami(__PACKAGE__), $_, $lexemeHashp->{value});
+		  }
+		  if ($_ eq 'IDENTIFIER') {
+		      $self->{_lastIdentifier} = $lexemeHashp->{value};
+		  }
+	      } else {
+		  if ($is_debug) {
+		      $log->debugf('[%s] Failed alternative %s "%s"', whoami(__PACKAGE__), $_, $lexemeHashp->{value});
+		  }
+	      }
 	  }
-	  if (! defined($self->{_impl}->lexeme_read($newlexeme, $lexemeHashp->{start}, $lexemeHashp->{length}, $lexemeHashp->{value}))) {
+	  if (! @alternativesOk) {
 	      my $line_columnp = lineAndCol($self->{_impl});
-	      logCroak("[%s] Lexeme value \"%s\" cannot be associated to lexeme name %s at position %d:%d.\n\nLast position:\n\n%s%s", whoami(__PACKAGE__), $lexemeHashp->{value}, $newlexeme, $lexemeHashp->{line}, $lexemeHashp->{column}, showLineAndCol(@{$line_columnp}, $self->{_sourcep}), $self->_context());
+	      logCroak("[%s] Lexeme value \"%s\" cannot be associated to %s at position %d:%d.\n\nLast position:\n\n%s%s", whoami(__PACKAGE__), $lexemeHashp->{value}, \@alternatives, $lexemeHashp->{line}, $lexemeHashp->{column}, showLineAndCol(@{$line_columnp}, $self->{_sourcep}), $self->_context());
 	  }
-          $lexemeHashp->{name} = $newlexeme;
+	  if (! defined($self->{_impl}->lexeme_complete($lexemeHashp->{start}, $lexemeHashp->{length}))) {
+	      my $line_columnp = lineAndCol($self->{_impl});
+	      logCroak("[%s] Lexeme value \"%s\" cannot be completed at position %d:%d.\n\nLast position:\n\n%s%s", whoami(__PACKAGE__), $lexemeHashp->{value}, $lexemeHashp->{line}, $lexemeHashp->{column}, showLineAndCol(@{$line_columnp}, $self->{_sourcep}), $self->_context());
+	  }
+          $lexemeHashp->{name} = $alternativesOk[0];
           $delta = $lexemeHashp->{length};
 	  #
 	  # A lexeme_read() can generate an event
@@ -603,7 +719,7 @@ MarpaX::Languages::C::AST - Translate a C source to an AST
 
 =head1 VERSION
 
-version 0.33
+version 0.36
 
 =head1 SYNOPSIS
 
@@ -657,6 +773,24 @@ Instantiate a new object. Takes as parameter an optional hash of options that ca
 
 Name of a grammar. Default is 'ISO-ANSI-C-2011'.
 
+=item typedef
+
+An array reference to a list of known typedefs, injected at top scope before parsing start. This option should I<not> be used unless you pass a C source that is incomplete. Typically something that has not gone through a preprocessor. Default is [] i.e. empty list.
+
+=item enum
+
+An array reference to a list of known enums, injected at top scope before parsing start. Alike for typedef, this option should I<not> be used unless you pass a C source that is incomplete. Typically something that has not gone through a preprocessor. Default is [] i.e. empty list.
+
+=item lazy
+
+A flag saying the parser to inject automatically all allowed alternatives when the grammar reaches a TYPEDEF_NAME/ENUMERATION_CONSTANT/IDENTIFIER ambiguity. This option should be used in practice only when you are parsing a source code not pre-processed. Please note that I<if> lazy mode is on, there might be several parse tree values. In such a case, unless the option $optionalArrayOfValuesb of the value() method is true, the first of the parse tree values will be returned. If more than one alternative is accepted, the lexemeCallback (see below) will be, in order of preference, either TYPEDEF_NAME, ENUMERATION_CONSTANT or IDENTIFIER. The lazy mode can produce more than one parse tree value. The options typedef and enum (see upper) can be used to help lazy mode choose between TYPEDEF_NAME and ENUMERATION_CONSTANT, while IDENTIFIER will always be pushed as an alternative. Default is a false value.
+
+=item start
+
+A string giving the starting point of the grammar. This should be used when you know that the source code to parse is not a full valid source, but a portion of if. This requires knowledge of the grammar rules. Default is empty string: '', i.e. let the grammar apply its default start rule.
+
+Please note that giving another value but 'translationUnit' will emit warnings from the grammar, saying that some rules are not reachable.
+
 =item logInfo
 
 Reference to an array of lexemes for which a log of level INFO will be issued.
@@ -711,6 +845,10 @@ Return the blessed value. Takes as optional parameter a flag saying if the retur
 
 Since version 0.30, the c2ast.pl script is named c2ast (i.e. without extension).
 
+=head1 NOTES
+
+C code can have inline ASM code. The GCC Inline Assembly is fully supported, any other is falling into a heuristic that should catch everything needed. CL inline assemblies have been targetted in particular.
+
 =head1 SEE ALSO
 
 L<Log::Any>, L<Marpa::R2>
