@@ -8,7 +8,7 @@ use IO::String;
 
 # ABSTRACT: ISO ANSI C 2011 grammar written in Marpa BNF
 
-our $VERSION = '0.33'; # TRIAL VERSION
+our $VERSION = '0.36'; # TRIAL VERSION
 
 
 our %DEFAULT_PAUSE = (
@@ -17,6 +17,7 @@ our %DEFAULT_PAUSE = (
     IDENTIFIER           => 'before',
     SEMICOLON            => 'after',
     LCURLY_SCOPE         => 'after',
+    LCURLY_REENTERSCOPE  => 'after',
     RCURLY_SCOPE         => 'after',
     COMMA                => 'after',
     EQUAL                => 'after',
@@ -28,7 +29,7 @@ our %DEFAULT_PAUSE = (
 our $DATA = do {local $/; <DATA>};
 
 sub new {
-  my ($class, $pausep) = @_;
+  my ($class, $pausep, $start) = @_;
 
   my $self  = {
     _grammar_option => {action_object  => sprintf('%s::%s', __PACKAGE__, 'Actions')},
@@ -50,6 +51,7 @@ sub new {
 
   $self->{_content} = '';
   my $allb = exists($pause{__ALL__});
+  $start ||= 'translationUnit';
   my $data = IO::String->new($DATA);
   while (defined($_ = <$data>)) {
       my $line = $_;
@@ -75,6 +77,8 @@ sub new {
       $self->{_content} .= $line;
   }
 
+  $self->{_content} =~ s/\$START\n/$start/;
+
   bless($self, $class);
 
   return $self;
@@ -110,7 +114,7 @@ MarpaX::Languages::C::AST::Grammar::ISO_ANSI_C_2011 - ISO ANSI C 2011 grammar wr
 
 =head1 VERSION
 
-version 0.33
+version 0.36
 
 =head1 SYNOPSIS
 
@@ -126,13 +130,13 @@ version 0.33
 
 =head1 DESCRIPTION
 
-This modules returns describes the ISO ANSI C 2011 C grammar written in Marpa BNF, as of L<http://www.quut.com/c/ANSI-C-grammar-y-2011.html> and L<http://www.quut.com/c/ANSI-C-grammar-l.html>.
+This modules contains the ISO ANSI C 2011 C grammar written in Marpa BNF, as of L<http://www.quut.com/c/ANSI-C-grammar-y-2011.html> and L<http://www.quut.com/c/ANSI-C-grammar-l.html>.
 
 =head1 SUBROUTINES/METHODS
 
-=head2 new([$pausep])
+=head2 new([$pausep, $start])
 
-Instance a new object. Takes a reference to a HASH for lexemes for which a pause after is requested.
+Instance a new object. Takes an eventual reference to a HASH for lexemes for which a pause after is requested, followed by an eventual start rule. Default paused lexemes is hardcoded to a list of lexeme that must always be paused, and this list cannot be altered. Default start rule is 'translationUnit'.
 
 =head2 content()
 
@@ -172,12 +176,12 @@ __DATA__
 # Defaults
 #
 :default ::= action => [values] bless => ::lhs
-lexeme default = action => [start,length,value]
+lexeme default = action => [start,length,value] forgiving => 1
 
 #
 # G1 (grammar), c.f. http://www.quut.com/c/ANSI-C-grammar-y-2011.html
 #
-:start ::= translationUnit
+:start ::= $START
 
 primaryExpression
 	::= IDENTIFIER
@@ -243,7 +247,7 @@ gccAlignofExpression ::= GCC_ALIGNOF unaryExpression
 
 unaryExpression
 	::= postfixExpression
-	| (gccExtension) postfixExpression
+	| gccExtension postfixExpression
 	| INC_OP unaryExpression
 	| DEC_OP unaryExpression
 	| unaryOperator castExpression
@@ -377,7 +381,7 @@ gccExtension ::= GCC_EXTENSION
 #                            | typeQualifier
 #                            | functionSpecifier
 #                            | alignmentSpecifier
-#                            | (gccExtension)
+#                            | gccExtension
 
 declarationSpecifiers ::= declarationSpecifiers0
                         | declarationSpecifiers1
@@ -392,8 +396,8 @@ declarationSpecifiers0 ::=       # List without type specifiers
                          | declarationSpecifiers0 functionSpecifier
                          | alignmentSpecifier
                          | declarationSpecifiers0 alignmentSpecifier
-                         | (gccExtension)
-                         | declarationSpecifiers0 (gccExtension)
+                         | gccExtension
+                         | declarationSpecifiers0 gccExtension
 
 declarationSpecifiers1 ::=       # List with a single typeSpecifier1
                            typeSpecifier1
@@ -402,7 +406,7 @@ declarationSpecifiers1 ::=       # List with a single typeSpecifier1
                          | declarationSpecifiers1 typeQualifier
                          | declarationSpecifiers1 functionSpecifier
                          | declarationSpecifiers1 alignmentSpecifier
-                         | declarationSpecifiers1 (gccExtension)
+                         | declarationSpecifiers1 gccExtension
 
 declarationSpecifiers2 ::=       # List with one or more typeSpecifier2
                            typeSpecifier2
@@ -412,7 +416,7 @@ declarationSpecifiers2 ::=       # List with one or more typeSpecifier2
                          | declarationSpecifiers2 typeQualifier
                          | declarationSpecifiers2 functionSpecifier
                          | declarationSpecifiers2 alignmentSpecifier
-                         | declarationSpecifiers2 (gccExtension)
+                         | declarationSpecifiers2 gccExtension
 
 # declarationSpecifiers ::= declarationSpecifiersUnit+
 
@@ -488,7 +492,7 @@ structDeclaration
 
 #specifierQualifierListUnit ::= typeSpecifier
 #                             | typeQualifier
-#                             | (gccExtension)
+#                             | gccExtension
 
 # specifierQualifierList ::= specifierQualifierListUnit+
 
@@ -499,21 +503,21 @@ specifierQualifierList ::= specifierQualifierList0
 specifierQualifierList0 ::= # List without type specifiers
                             typeQualifier
                           | specifierQualifierList0 typeQualifier
-                          | (gccExtension)
-                          | specifierQualifierList0 (gccExtension)
+                          | gccExtension
+                          | specifierQualifierList0 gccExtension
 
 specifierQualifierList1 ::= # List with a single typeSpecifier1
                             typeSpecifier1
                           | specifierQualifierList0 typeSpecifier1
                           | specifierQualifierList1 typeQualifier
-                          | specifierQualifierList1 (gccExtension)
+                          | specifierQualifierList1 gccExtension
 
 specifierQualifierList2 ::= # List with one or more typeSpecifier2
                             typeSpecifier2
                           | specifierQualifierList0 typeSpecifier2
                           | specifierQualifierList2 typeSpecifier2
                           | specifierQualifierList2 typeQualifier
-                          | specifierQualifierList2 (gccExtension)
+                          | specifierQualifierList2 gccExtension
 
 structDeclaratorList
 	::= structDeclarator
@@ -571,6 +575,10 @@ declarator
         #
         | MSVS___C_ASSERT__ LBRACKET expression RBRACKET
 
+#
+# It is VERY important that directDeclaratorIdentifier remains forever an LHS
+# with a single RHS: IDENTIFIER, c.f. comment in AST.pm
+#
 event 'directDeclaratorIdentifier$' = completed <directDeclaratorIdentifier>
 directDeclaratorIdentifier
 	::= IDENTIFIER
@@ -746,8 +754,8 @@ externalDeclaration
 	::= functionDefinition
 	| declaration
 
-compoundStatementReenterScope ::= LCURLY RCURLY_SCOPE
-	                        | LCURLY blockItemList RCURLY_SCOPE
+compoundStatementReenterScope ::= LCURLY_REENTERSCOPE RCURLY_SCOPE
+	                        | LCURLY_REENTERSCOPE blockItemList RCURLY_SCOPE
 
 functionDefinition
 	::= functionDefinitionCheck1
@@ -1022,28 +1030,48 @@ EQ_OP        ~ '=='
 NE_OP        ~ '!='
 :lexeme ~ <SEMICOLON>     priority => -126
 SEMICOLON                     ~ ';'
-:lexeme ~ <LCURLY>        priority => -127
-LCURLY                       ~ '{' | '<%'
+#
+# LCURLY factorization
+#
+_LCURLY      ~ '{' | '<%'
+:lexeme ~ <LCURLY>              priority => -127
 :lexeme ~ <LCURLY_SCOPE>        priority => -127
-LCURLY_SCOPE                       ~ '{' | '<%'
+:lexeme ~ <LCURLY_REENTERSCOPE> priority => -127
+LCURLY              ~ _LCURLY
+LCURLY_SCOPE        ~ _LCURLY
+LCURLY_REENTERSCOPE ~ _LCURLY
+#
+# LCURLY factorization
+#
+_RCURLY ~ '}' | '%>'
 :lexeme ~ <RCURLY>        priority => -128
-RCURLY                       ~ '}' | '%>'
 :lexeme ~ <RCURLY_SCOPE>        priority => -128
-RCURLY_SCOPE                       ~ '}' | '%>'
+RCURLY       ~ _RCURLY
+RCURLY_SCOPE ~ _RCURLY
+
 :lexeme ~ <COMMA>         priority => -129
 COMMA                     ~ ','
 :lexeme ~ <COLON>         priority => -130
 COLON                      ~ ':'
 :lexeme ~ <EQUAL>         priority => -131
 EQUAL       ~ '='
+#
+# LPAREN factorization
+#
+_LPAREN ~ '('
 :lexeme ~ <LPAREN>        priority => -132
-LPAREN                ~ '('
 :lexeme ~ <LPAREN_SCOPE>        priority => -132
-LPAREN_SCOPE                ~ '('
+LPAREN       ~ _LPAREN
+LPAREN_SCOPE ~ _LPAREN
+#
+# RPAREN factorization
+#
+_RPAREN ~ ')'
 :lexeme ~ <RPAREN>        priority => -133
-RPAREN                      ~ ')'
 :lexeme ~ <RPAREN_SCOPE>        priority => -133
-RPAREN_SCOPE                      ~ ')'
+RPAREN       ~ _RPAREN
+RPAREN_SCOPE ~ _RPAREN
+
 :lexeme ~ <LBRACKET>      priority => -134
 LBRACKET      ~ '[' | '<:'
 :lexeme ~ <RBRACKET>      priority => -135
@@ -1335,7 +1363,6 @@ opaqueAsmStatement ::= ANY_ASM ASM_OPAQUE
 
 ###############################################################################################
 # Discard simple preprocessor directives (on one line - cpp output persist to get some of them)
-# Too bad if AST.pm did not catched it after a pause lexeme
 ###############################################################################################
 <Cpp style directive start> ~ '#'
 <Cpp style directive interior single line> ~ [^\n]*
