@@ -9,7 +9,7 @@ use MarpaX::Languages::C::AST::Util qw/whoami/;
 use Log::Any qw/$log/;
 use Carp qw/croak/;
 
-our $VERSION = '0.33'; # TRIAL VERSION
+our $VERSION = '0.36'; # TRIAL VERSION
 
 
 sub new {
@@ -19,7 +19,7 @@ sub new {
       _nscope => 0,
       _typedefPerScope => [ {} ],
       _enumAnyScope => {},
-      _delay => 0,
+      _delay => [],
       _enterScopeCallback => [],
       _exitScopeCallback => [],
   };
@@ -59,6 +59,7 @@ sub parseEnterScope {
   # Doing \%{$...} is just to make sure this is a new hash instance, with keys pointing to the same values as the origin
   #
   push(@{$self->{_typedefPerScope}}, \%{$self->{_typedefPerScope}->[$self->{_nscope}]});
+  push(@{$self->{_delay}}, 0);
   $self->{_nscope}++;
 
   if (@{$self->{_enterScopeCallback}}) {
@@ -76,9 +77,9 @@ sub parseDelay {
     if ($log->is_debug) {
 	$log->debugf('[%s] Setting delay flag to %d at scope %d', whoami(__PACKAGE__), $value, $self->{_nscope});
     }
-    $self->{_delay} = $value;
+    $self->{_delay}->[-1] = $value;
   }
-  return $self->{_delay};
+  return $self->{_delay}->[-1];
 }
 
 
@@ -142,13 +143,27 @@ sub doExitScope {
       $log->debugf('[%s] Removing scope %d', whoami(__PACKAGE__), $self->{_nscope});
   }
   pop(@{$self->{_typedefPerScope}});
+  pop(@{$self->{_delay}});
   $self->{_nscope}--;
 
   if (@{$self->{_exitScopeCallback}}) {
       my ($ref, @args) = @{$self->{_exitScopeCallback}};
       &$ref(@args);
   }
-  $self->parseDelay(0);
+
+  if ($self->{_nscope} > 0) {
+    #
+    # If the parent scope was marked delayed, we close it as well:
+    #
+    if ($self->parseDelay) {
+      if ($log->is_debug) {
+        $log->debugf('[%s] Parent scope has delay flag on', whoami(__PACKAGE__));
+      }
+      $self->doExitScope;
+    } else {
+      $self->parseDelay(0);
+    }
+  }
 }
 
 
@@ -235,7 +250,7 @@ MarpaX::Languages::C::AST::Scope - Scope management when translating a C source
 
 =head1 VERSION
 
-version 0.33
+version 0.36
 
 =head1 SYNOPSIS
 
@@ -285,7 +300,7 @@ Say we enter a scope.
 
 =head2 parseDelay($self, [$value])
 
-Returns/Set current delay flag.
+Returns/Set delay flag of the current (i.e. last) scope.
 
 =head2 parseScopeLevel($self)
 
